;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Menu Controller
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,;;
.include "cpctelera.h.s"
.include "cpctelera_functions.h.s"
.include "menu_controller.h.s"
.include "Managers/game_manager.h.s"
.include "Systems/colision_system.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Define Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
string: .asciz "Pulsa 'x' para jugar.";
string2: .asciz "                      ";
boolean: .db #0


.module menu_controller

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;++++++++++++++++++++++++++++++++
;; M_Menu_Init
;;--------------------------------
;;	Draw a string before you 
;; start the game. (we change this
;; for draw an image that will be 
;; our menu)
;; Input: -
;; Destroys: AF, DE, BC,IY, HL
;;

m_menu_init::

	;; clear the fireware
	ld hl, #0xC000
	ld de, #0XC001
	ld (hl), #0x00
	ld bc, #0x4000
	ldir


	ld a, #0
	ld (boolean), a


	;;Set up draw char colours before calling draw string
	ld	d, #0	;; D = Background PEN (16)
	ld	e, #2		;; E = Foreground PEN (3)
;;<<<<<<<<<<<<<< Mode 1 >>>>>>>>>>>>>>>>>>>>>
	call cpct_setDrawCharM1_asm	;;Set draw char colours

	;;Calculate a video-memory location for printing a string
	ld   de, #CPCT_VMEM_START_ASM	;; DE = Pointer to start of the screen
	ld	b, #24			;; B = y coordinate (24 = 0x18)
	ld	c, #16			;; C = x coordinate (16 = 0x10)

	call cpct_getScreenPtr_asm	;; Calculate video memory location and return it in HL

	;; Print the srting in video memory
	;; HL already points to video memory, as it is the return
	;; vvalue from cpct_getscreenPtr_asm
	ld 	iy, #string		;; IY = Pointer to the string

;;<<<<<<<<<<<<<< Mode 1 >>>>>>>>>>>>>>>>>>>>>
	call cpct_drawStringM1_asm

	ret

;;++++++++++++++++++++++++++++++++++
;; M_Controller_Play
;;----------------------------------
;;	Scan the keyboard for start the
;; game when you press the X.
;; Input: -
;; Destroys: AF, BC, DE, HL, IY
;;

m_controller_play::
	;; Check the boolean 
	ld 	a, (boolean)	;; When the boolean is 1 the game had been start
	cp	#1			;; and you don't need to check all other time.
	jr	z, renitem 

	call cpct_scanKeyboard_f_asm	;; Scan the keyboard for find the key pressed

	ld 	hl, #Key_X		;; Set the X key id in HL
	call cpct_isKeyPressed_asm	;; Compare the key pressed with HL
	jr	z, X_NotPressed

X_Pressed:
	ld	a, #1
	ld	(boolean), a	;; Set the a value to boolean
	;;Calculate a video-memory location for printing a string
	ld   de, #CPCT_VMEM_START_ASM	;; DE = Pointer to start of the screen
	ld	b, #24			;; B = y coordinate (24 = 0x18)
	ld	c, #16			;; C = x coordinate (16 = 0x10)

	call cpct_getScreenPtr_asm	;; Calculate video memory location and return it in HL

	;; Print the srting in video memory
	;; HL already points to video memory, as it is the return
	;; vvalue from cpct_getscreenPtr_asm
	ld 	iy, #string2		;; IY = Pointer to the string

;;<<<<<<<<<<<<<< Mode 1 >>>>>>>>>>>>>>>>>>>>>
	call cpct_drawStringM1_asm

	call man_game_render

X_NotPressed:

	ret

renitem:

	ret

;;+++++++++++++++++++++++++++++++
;; M_controller_boolean_value
;;--------------------------------
;;	Return de value of the boolean.
;; Input: -
;; Destroys: AF
;; Return:
;;	A = the boolean value
;;
m_controller_boolean_value::

	ld a, (boolean)

	ret
