;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Entity_Manager
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "entity_manager.h.s"
.include "array_structures.s"
.include "cpctelera.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Define Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
max_entities == 5

DefineComponentArrayStructure _entity, max_entities, DefineCmp_Entity_Default

ent1: DefineCmp_Entity 40, 180, 0, 0, 4, 16, _sp_plant3, e_ai_st_noAI, 0, 1
bullet1: DefineCmp_Entity 40, 140, 0, 0, 4, 16, _sp_bullet, e_ai_st_noAI, 0, 1
ent2: DefineCmp_Entity 4, 120, 2, 1, 4, 16, _sp_bugui, e_ai_st_AI_appear, 1, 0
ent3: DefineCmp_Entity 4, 10, 2, 4, 4, 16, _sp_bugui, e_ai_st_AI_appear, 2, 0
ent4: DefineCmp_Entity 50, 12, 1, 1, 4, 16, _sp_bugui, e_ai_st_AI_appear, 0, 0
ent5: DefineCmp_Entity 50, 120, 1, 1, -1, 16, _sp_plant3, e_ai_st_AI_appear, 0, 0
ent6: DefineCmp_Entity 5, 125, 1, 1, 1, 16, _sp_plant3, e_ai_st_AI_appear, 0, 0

.module entity_manager


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;+++++++++++++++++++++++++++++++
;; Man_Entity_GetArray
;;-------------------------------
;;	Gets a pointer to the array of 
;; entities in IX and also the number
;; of entities in A.
;; Input: -
;; Destroys: AF, IX
;; Returns:
;;	A: number of entities in the array
;;	IX: pointer to the start of the array
;;
man_entity_getArray::
	ld 	ix, #_entity_array	;; get the array memory adress and put it in IX
	ld 	a, (_entity_num)	;; get the number of entities

	ret

;;+++++++++++++++++++++++++++++++
;; Man_Entity_GetArrayHL
;;---------------------------------
;;	Return the pointer to the array
;; entities in HL.
;; Input: -
;; Destroys: HL
;; Returns: 
;;	HL = pointer to the entity array
;;
man_entity_getArrayHL::
	ld 	hl, #_entity_array	;; get the array memory adress and put it in IX
		;; get the number of entities

	ret

;;++++++++++++++++++++++++++++++++++++++++
;; Man_Entity_Init
;;----------------------------------------
;;	Initialize the entity manager. IT set
;; ups everything with 0 entities and reaty 
;; to start creating new ones.
;; Input: -
;; Destroys: AF, DE, HL
;;
man_entity_init::
	;;Reset all component vector values
	xor 	a
	ld 	(_entity_num), a	;; set the a value in number of entities

	ld 	hl, #_entity_array	;; get the array memory adress and put it in IX
	ld 	(_entity_pend), hl	;; set the hl value in entity pend

	;; First entity must be marked as no entity
	;; as it is the last valid entity in the array
	ex 	de, hl			;; save hl into de
	ld 	hl, #e_w			;; hl offset of e_w_in_entity
	add	hl, de			;; hl entity + e_w
	ld	(hl), #e_w_invalidEntity	;; entity[0]
	ex	de, hl			;; restore hl

	ret


;;++++++++++++++++++++++++++++++++++++++++
;; Man_Entity_New
;;-----------------------------------------
;;	Add a new entity component to the array 
;; without initializing it. It does not perform
;; any check for space in the array.
;; Input: -
;; Destroys: AF, BC, DE, HL
;; Return:
;;	DE =  points to added element
;;	BD = sizeof (entity_t)
;;
man_entity_new::
	;;increment number of reserved entities
	ld 	hl, #_entity_num
	inc 	(hl)

	;;increment array end pointer to point to the next
	;;free element in the array
	ld 	hl, (_entity_pend)
	ld 	d, h
	ld 	e,l
	ld 	bc, #sizeof_e
	add 	hl, bc
	ld 	(_entity_pend), hl

	ret 


;;+++++++++++++++++++++++++++++++++++++++++++
;; Man_Entity_Create
;;------------------------------------------
;;	Creates and initializes new entity.
;;	Returns a pointer to newly created entity
;; in IX.
;; Input: 
;;	HL = pointer to initializer values for the 
;;	     entity to be created.
;; Destroys: AF, BC, HL, DE
;; Return:
;;	IX = pointer to the component created
;;
man_entity_created::
	push 	hl
	call man_entity_new

	;;IX == DE (IX returns the value of the pointer to the entity created)
	ld__ixh_d
	ld__ixl_e

	;; Copy initialization values to new entity
	;; DE points to the new added entity
	;; BC holds sizeof(Entity_t)
	;; Copy initialization values
	pop 	hl
	ldir

	ret


;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity1HL::
	ld 	hl, #ent1	;; set the entity memory adress in hl

	ret

;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity2HL::
	ld 	hl, #ent2	;; set the entity memory adress in hl

	ret

;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity3HL::
	ld 	hl, #ent3	;; set the entity memory adress in hl

	ret

;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity4HL::
	ld 	hl, #ent4	;; set the entity memory adress in hl

	ret


;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity5HL::
	ld 	hl, #ent5	;; set the entity memory adress in hl

	ret

;;++++++++++++++++++++++++++++++++++++++++
;; Man Entity_GetEntity1HL
;;--------------------------------------
;;	Return in hl the memory adress of the 
;; entity.
;; Input: -
;; Destroys: HL
;; Return:
;;	HL = entity memory adress
;;
man_entity_getEntity6HL::
	ld 	hl, #ent6	;; set the entity memory adress in hl

	ret

;;+++++++++++++++++++++++++++++++++++++++
;; Man_Entity_Get_Entity_Bullet
;;---------------------------------------
;;	Returns in HL the memory adress of the
;; bullet entity.
;; Input: -
;; Destroys: HL
;; Returns:
;;	HL = pointer to the bullet entity
;;
man_entity_getEntityBullet::
	ld 	hl, #bullet1	;; set the entity memory adress in hl

	ret

;;+++++++++++++++++++++++++++++++++++
;; Man_Entity_GetEntityBulletIX
;;----------------------------------
;;	Returns in IX pointer to the bullet
;; entity in entity_array.
;; Input: -
;; Destroys: AF, DE, IX
;; Returns: 
;;	IX = pointer to the bullet entity
;;
man_entity_getEntityBulletIX::
	call man_entity_getArray	;; set the entity memory adress in hl
	ld de, #sizeof_e
	add ix, de
	ret

;;++++++++++++++++++++++++++++++++++++
;; Man_Entity_GetEntityBulletIY
;;----------------------------------
;;	Returns in IY pointer to the bullet
;; entity in entity_array.
;; Input: -
;; Destroys: AF, DE, IX, IY
;; Returns:
;;	IY = pointer to the bullet entity
;;
man_entity_getEntityBulletIY::
	call man_entity_getArray	;; set the entity memory adress in hl
	ld de, #sizeof_e
	add ix, de
	push ix
	pop iy
	ret

;;++++++++++++++++++++++++++++++++++
;; Man_Entity_GetEntityEnemyIX
;;----------------------------------
;;	Return in IX pointer to first enemy
;; entity in entity_array.
;; Input: -
;; Destroys: AF, DE, IX
;; Returns:
;;	IX = pointer to the first enemy entity
;;
man_entity_getEntityenemyIX::
	call man_entity_getEntityBulletIX	;; set the entity memory adress in hl
	ld de, #sizeof_e
	add ix, de
	ret


