;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Game Manager
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera_functions.h.s"
.include "Systems/render_system.h.s"
.include "game_manager.h.s"
.include "entity_manager.h.s"
.include "cpctelera.h.s"
.include "Systems/input_system.h.s"
.include "Systems/physics_system.h.s"
.include "Controllers/menu_controller.h.s"
.include "Systems/ia_system_patrol.h.s"
.include "Managers/patrol_manager.h.s"
.include "Systems/colision_system.h.s"



.module game_manager


;string: .asciz "Pulsa 'x' para jugar.";


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;+++++++++++++++++++++++++++++++++++++
;; Man_Game_Init
;;-------------------------------------
;;	Initialize the game, all the 
;;entities and all the variables
;; Input: -
;; Destroys: AF, BC, DE, HL, IX
;;
man_game_init::
	;; Set the videomode to 0
	ld	c, #0
	call cpct_setVideoMode_asm

	;; Use the palette that you define
	ld    hl,   #_palette
   	ld    de,   #16
   	call  cpct_setPalette_asm

	;; Init Managera
	call man_entity_getArray
	call man_patrol_init
	call man_entity_init

	;; Init Systems
	call man_entity_getArray

	call sys_ai_control_init
	call sys_render_init
	call sys_physics_init
	call sys_input_init


	;; Create the entities
	call man_entity_getEntity1HL
	call man_entity_created

	call man_entity_getEntityBullet
	call man_entity_created

	call man_entity_getEntity2HL
	call man_entity_created

	call man_entity_getEntity3HL
	call man_entity_created

	call man_entity_getEntity4HL
	call man_entity_created

	ret

;;++++++++++++++++++++++++++++++++++++++
;; Man_Game_Update
;;--------------------------------------
;;	Update 1 Game Cycle doing everithing 
;;except the rendering. Rendering is 
;;considered apart for its time constraints
;; Input: -
;; Destroys: AF, BC, DE, IX
;;
man_game_update::

	;; Update input and physics systems
	call man_entity_getArray
	call sys_input_update

	call m_controller_boolean_value
   	cp    #0
   	jr    z, no_physics

	call man_entity_getArray
	call sys_ai_control_update

	call man_entity_getArray
	call sys_physics_update

	call sys_colision_controller

no_physics: 

	ret

;;+++++++++++++++++++++++++++++++++++++++++
;; Man_Game_Render
;;------------------------------------------
;; 	Does rendering process apart from
;;game update to account for its time 
;;constraints.
;; Input:-
;; Destroys: AF, BC, DE, HL, IX
;;
man_game_render::
	;; draw the entities
	call man_entity_getArray
	call sys_render_update

	ret
