;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	AI System Patrol
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "Managers/entity_manager.h.s"
.include "Managers/patrol_manager.h.s"

.module sys_ai_control
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;+++++++++++++++++++++++++++++++
;; Sys_AI_Control_Init
;;--------------------------------
;;	Initialize the ai control system
;; Input: -
;;	IX = pointer to the entity array
;;
sys_ai_control_init::
	ld (_ent_array_ptr_temp_appear), ix

	ld (_ent_array_ptr), ix

	ret

;;++++++++++++++++++++++++++++++++
;; Sys_AI_Control_Update
;;--------------------------------
;;	Updates all AI entities.
;; Input:
;;	IX = pointer to the entity array
;;	A = number of elements in the array
;; Destroys: AF, BC, DE, IX
;;
sys_ai_control_update::
	ld	(_ent_counter), a

	_ent_array_ptr = . + 2
	ld 	ix, #0x0000

_loop:
	ld	a, e_ai_st(ix)
	cp	#e_ai_st_noAI
	jr	z, _no_AI_ent

_AI_ent:
	cp 	#e_ai_st_AI_appear
	call	z, sys_ai_appear
	cp	#e_ai_st_AI_move
	call	z, sys_ai_move 
	cp	#e_ai_st_AI_patrol
	call	z, sys_ai_patrol

_no_AI_ent:
_ent_counter = . +1
	ld 	a, #0
	dec 	a
	ret	z

	ld	(_ent_counter), a
	;; jump to next entity
	ld 	de, #sizeof_e
	add	ix, de

	jr	_loop

;;+++++++++++++++++++++++++++++++++++
;; Sys_AI_Appear
;;------------------------------------
;;	Set the position to appear the anemy.
;; Input: 
;;	IX = pointer to the entity
;; Destroys: -
;;
sys_ai_appear::
_ent_array_ptr_temp_appear = .+2

	ld 	e_ai_pre_st(ix), #e_ai_st_AI_appear
	ld 	e_ai_st(ix), #e_ai_st_AI_patrol

;;+++++++++++++++++++++++++++++++++
;; Sys_AI_Move
;;----------------------------------
;;	Set the movement of the enemies.
;; Input: 
;;	IX = pointer to the entity
;; Destroys: AF, IX
;;
sys_ai_move::
	ld	e_vx(ix), #0
	ld	e_vy(ix), #0

	ld	a, e_ai_aim_x(ix)	;; A = x objetivo
	sub	e_x(ix)		;; A = x_objetivo - x
	jr	nc, _objx_greater_or_equal	;; if (x_objetivo - x > 0) objx_greater

_objx_lesser:
	ld	e_vx(ix), #-1
	jr	endif_x

_objx_greater_or_equal:
	jr	z, arrived_x

	ld	e_vx(ix), #1
	jr	endif_x

arrived_x:
	ld	e_vx(ix), #0

endif_x:
	;;update y
	ld	a, e_ai_aim_y(ix)	;; A = x objetivo
	sub	e_y(ix)		;; A = x_objetivo - y
	jr	nc, _objy_greater_or_equal	;; if (y_objetivo - y > 0) objy_greater

_objy_lesser:
	ld	e_vy(ix), #-1
	jr	endif_y

_objy_greater_or_equal:
	jr	z, arrived_y

	ld	e_vy(ix), #1
	jr	endif_y

arrived_y:
	ld	e_vy(ix), #0
	ld	a, e_vx(ix)
	or 	a
	jr	nz, endif_y
	;;	X and y are both 0
	ld	a, e_ai_pre_st(ix)
	ld 	e_ai_st(ix), a
	ld 	e_ai_pre_st(ix), #e_ai_st_AI_move

endif_y:

	ret

;;+++++++++++++++++++++++++++++++++
;; Sys_AI_patrol
;;----------------------------------
;;	Set the movement of the enemies.
;; Input: -
;; Destroys: AF, HL
;;
sys_ai_patrol::
	ld 	l, e_ai_patrol_step_l(ix)
	ld 	h, e_ai_patrol_step_h(ix)

;;	comprobar x = -1	
	ld	a, (hl)	;; a = x
	cp	#patrol_invalid_move_x
	jr	z, _reset_patrol

	;; Set AI AIM(x, y)
	ld	e_ai_aim_x(ix), a
	inc	hl		;; point to y
	ld	a, (hl)	;; a = y
	ld	e_ai_aim_y(ix), a

	;; Set e_ai_patrol_step + 2
	inc 	hl
	ld 	e_ai_patrol_step_l(ix), l
	ld 	e_ai_patrol_step_h(ix), h

	;; change to move
	ld 	e_ai_pre_st(ix), #e_ai_st_AI_patrol
	ld 	e_ai_st(ix), #e_ai_st_AI_move

	ret

_reset_patrol:
	;; get next 2 bytes pointed by hl, which
	;; point to the start of the patrol
	inc 	hl
	ld	a, (hl)
	inc 	hl
	ld	h, (hl)

	;; reset patrol step
	ld 	e_ai_patrol_step_l(ix), a
	ld 	e_ai_patrol_step_h(ix), h

	ret