;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Physics System
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "Managers/entity_manager.h.s"
.include "physics_system.h.s"
.include "Managers/bullet_manager.h.s"

.module sys_entity_physics
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Define Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
screen_width = 80
screen_height = 200

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;++++++++++++++++++++++++++++++++++
;; Sys_Physics_Init
;;------------------------------------
;;	Inits the render system.
;; Input: -
;; Destroys: -
;;
sys_physics_init::
	ret

;;++++++++++++++++++++++++++++++++++++++
;; Sys_Physics_Update
;;--------------------------------------
;;	Updates all physics components of 
;; entities.
;;	Assumes that entities are contiguous
;; and all values.
;;	Assumes there is at least one entity
;; in the array.
;; Input:
;;	IX = pointer to the entity array
;;	A = number of elements in the array
;; Destroys: AF, BC, DE, IX
;; 
sys_physics_update::
	;; There are elements, lets update them
	ld 	b, a

update_loop:
	;; update x

	ld 	a, #screen_width + 1
	sub 	e_w(ix)	;; subtrack the width of the entity
	ld 	c, a

	ld 	a, e_x(ix)
	add 	e_vx(ix)	;; plus the velocity to the x coordinate
	cp 	c
	jr 	nc, invalid_x	;; if x coordinate is over to the screen go to invalid_x

valid_x:
	ld	e_x(ix), a	;; change de x coordinate
	jr	endif_x

invalid_x:
	ld 	a, e_vx(ix)
	neg
	ld 	e_vx(ix), a	;; the entity bounce of to the x screen limit

endif_x:
	;; update y
	ld 	a, #screen_height + 1
	sub 	e_h(ix)	;; subtrack the height of the entity
	ld	c, a

	ld 	a, e_y(ix)
	add 	e_vy(ix)	;; plus the velocity to the y coordinate
	cp	c
	jr	nc, invalid_y	;; if y coordinate is over to the screen go to invalid_y

valid_y:
	ld	e_y(ix), a	;; change de y coordinate
	jr endif_y

invalid_y:
	call	man_bullet_resetPosition
	ld 	a, e_vy(ix)
	neg
	ld	e_vy(ix), a	;; the entity bounce of to the y screen limit

endif_y:
	dec 	b	;; gdecrement in 1
	ret 	z

	ld 	de, #sizeof_e
	add 	ix, de	;; go to the next entity
	jr 	update_loop
