;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Render System
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Include Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera_functions.h.s"
.include "cpctelera.h.s"
.include "render_system.h.s"
.include "Managers/entity_manager.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Define Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module sys_entity_render

screen_start = 0xC000  ;;video memory start


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function Section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;++++++++++++++++++++++++++++++++++++++
;; Sys_Render_Init
;;--------------------------------------
;;	Inits the render_system
;; Input: -
;; Destroys: -
;;
sys_render_init::
	;; Set the videomode to 0
	ld	c, #0
	call cpct_setVideoMode_asm

	cpctm_setBorder_asm HW_WHITE	;; Set de border

	ret


;;++++++++++++++++++++++++++++++++++++++++
;; Sys_Render_Update
;;----------------------------------------
;;	Updates the render system, draws all 
;; entity components, assumes that entities 
;; are contiguous and all assumes there is at 
;; least one entity to render.
;; Input:
;;	IX = Pointer to entity array
;;	A = nunber of elements in the array
;; Destroys: AF, HL, BC, De, IX
;;
sys_render_update::
	;;render entities
	call sys_render_entities

	ret


;;+++++++++++++++++++++++++++++++++++++++++++
;; Sys_Render_Entities
;;-----------------------------------------
;;	Draws aññ emtoty components assumes that
;; entities are contiguous and all assumes there
;; is at least one entity to render.
;; Input: 
;;	IX = pointer to the first entity to render
;;	A = number of entities to render
;; Destroys: AF, HL, BC, DE, IX
;;
sys_render_entities::

	ld 	(_ent_counter), a

_update_loop:


	ld 	e, e_lastVP_l(ix)	;; hl points to las vidememorypointer
	ld 	d, e_lastVP_h(ix)
	xor	a
	ld 	c, e_w(ix)	;; width
	ld	b, e_h(ix)	;; height
	push	bc		;; save wodth - height pair
	call cpct_drawSolidBox_asm

	;;calculate the video memory
	ld 	de, #screen_start	;; points to the start of the screen
	ld 	c, e_x(ix)		;; x
	ld 	b, e_y(ix)		;; y
	call cpct_getScreenPtr_asm

	;;store video memeory pointer as last
	ld 	e_lastVP_l(ix), l	;; store hl as last video memory pointer
	ld 	e_lastVP_h(ix), h

	;;draw entity sprite
	ex 	de, hl
	ld 	l, e_pspr_l(ix)	;; points to the sprite
	ld 	h, e_pspr_h(ix)
	pop 	bc
	call cpct_drawSprite_asm
	;call cpct_drawSolidBox_asm

_ent_counter = .+1
	ld 	a, #0
	dec 	a
	ret 	z

	ld 	(_ent_counter), a
	ld 	bc, #sizeof_e
	add 	ix, bc
	jr _update_loop
