;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of PlantAttack: An Amstrad CPC Game.
;; 
;;
;;  This program is free software: you can redistribute it and/or
;;  modify it under the terms of the  GNU General Public License (GPLv3).
;;  
;;  If you need more information about the license GNU you can visit:
;;  https://www.gnu.org/licenses/quick-guide-gplv3.html


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Generate data structure and array of type-T components
;; 	Macro to generate all the data structure required
;; 	to manage an array of components of the same type
;; It generates these labels.
;;	*T_num: a byte to count the number of elements in the array
;;	*T_plast: a word to store a pointer to the end of the array
;;		this will be the first free element in the array
;;	*T_array: the array itself
;; Inputs:
;;	_Tname: name of the component type 
;;	_N: size of the array in number of components (to reserve)
;; 	DefinteTypeMacroDefault: macro to be called to generate a default
;;
.macro DefineComponentArrayStructure _Tname, _N, _DefineTypeMacroDefault
	_Tname'_num: .db 0  ;; number of defined elements
	_Tname'_pend: .dw _Tname'_array ;;pointer to the end of the array
	_Tname'_array::  ;; array of elements of the
	.rept _N
		_DefineTypeMacroDefault  ;; Repeatedly call default
	.endm
.endm

;;
;; Same macro as before, but generating a bunch of zeros instead of default
;; It uses _ComponentSize instead of _DefineTypeMacroDefault to produce
;; Inputs:
;;	Same as before except,
;; 	_componentSize: size in bytes of one T-component
;;
.macro DefineComponentArrayStructure_Size _Tname, _N, _ComponentSize
	_Tname'_num: .db 0  ;;number of defined elements
	_Tname'_pend: .dw _Tname'_array  ;; pointer to the end of the 
	_Tname'_array:   ;;array of elements to the
		.ds _N * _ComponentSize  ;;produce as many 0's as
.endm